package com.antai.sport.http.server.server.api.business.club.service;

import com.antai.sport.http.server.common.exception.BusinessException;
import com.antai.sport.http.server.repository.club.entity.Club;
import com.antai.sport.http.server.repository.club.mapper.ClubMapper;
import com.antai.sport.http.server.server.api.business.club.converter.ClubConverter;
import com.antai.sport.http.server.server.api.business.club.dto.CreateClubReqDTO;
import com.antai.sport.http.server.server.api.business.club.dto.UpdateClubReqDTO;
import com.antai.sport.http.server.server.api.business.club.mapper.ClubBusinessMapper;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;

@Service
public class ClubService {

    @Resource
    private ClubMapper clubMapper;

    @Resource
    private ClubBusinessMapper clubBusinessMapper;

    @Resource
    private ClubConverter clubConverter;

    /**
     * 创建俱乐部
     *
     * @param dto
     * @return
     */
    public Long create(CreateClubReqDTO dto) {
        if (clubBusinessMapper.checkChairman(dto.getCreateUser())) {
            throw new BusinessException("您已经是俱乐部会长，不能创建多个俱乐部");
        }
        checkClubNameRepeat(null, dto.getName());
        Club club = clubConverter.createDTOtoClub(dto, LocalDateTime.now());
        clubMapper.insert(club);
        return club.getId();
    }

    /**
     * 修改俱乐部信息
     *
     * @param dto
     */
    public void update(UpdateClubReqDTO dto) {
        checkClubNameRepeat(dto.getId(), dto.getName());
        Club club = clubConverter.updateDTOtoClub(dto, LocalDateTime.now());
        clubMapper.updateById(club);
    }

    /**
     * 校验俱乐部名称是否重复
     *
     * @param id
     * @param name
     */
    private void checkClubNameRepeat(Long id, String name) {
        if (clubBusinessMapper.checkClubNameRepeat(id, name)) {
            throw new BusinessException("俱乐部名称重复，请换一个名称试试");
        }
    }
}
